open! Dune_engine
open! Stdune
open Dune_file

val foreign_rules :
     Foreign.Library.t
  -> sctx:Super_context.t
  -> expander:Expander.t
  -> dir:Path.Build.t
  -> dir_contents:Dir_contents.t
  -> unit

val rules :
     Library.t
  -> sctx:Super_context.t
  -> dir_contents:Dir_contents.t
  -> dir:Path.Build.t
  -> expander:Expander.t
  -> scope:Scope.t
  -> Compilation_context.t * Merlin.t
