(** This command line utility is used to implement portable commands used in the
    cram test. For example, it's used to implement a portable version of stat.

    This command is invoked like this:

    [$ dune_cmd <subcommand> arguments] *)
