Generation of opam files is attached to @all
--------------------------------------------
Reproduction case for #2927

  $ cat >dune-project <<EOF
  > (lang dune 2.0)
  > (generate_opam_files true)
  > (package (name foo))
  > EOF

  $ dune build

  $ cat foo.opam
  # This file is generated by dune, edit dune-project instead
  opam-version: "2.0"
  depends: [
    "dune" {>= "2.0"}
  ]
  build: [
    ["dune" "subst"] {pinned}
    [
      "dune"
      "build"
      "-p"
      name
      "-j"
      jobs
      "@install"
      "@runtest" {with-test}
      "@doc" {with-doc}
    ]
  ]
