Expected: a, b, ..., j (required) and optional (optional)

  $ dune external-lib-deps @all
  These are the external library dependencies in the default context:
  - a
  - b
  - c
  - d
  - e
  - f
  - h
  - i
  - j
  - optional (optional)
